package dk.nsi.dds.projects.sxa.common.security;

import static org.junit.Assert.assertEquals;

import java.io.IOException;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;

import org.apache.cxf.binding.soap.SoapFault;
import org.apache.http.HttpStatus;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.runners.MockitoJUnitRunner;
import org.springframework.mock.web.MockHttpServletResponse;

import dk.nsi.dds.projects.sxa.common.configuration.SXAConfigKeys;
import dk.nsi.util.Config;
import dk.nsi.util.ConfigException;

@RunWith(MockitoJUnitRunner.class)
public class AccessValidationFilterTest {
	@InjectMocks
	private AccessValidationFilter subject;

	@Mock
	private Config config;

	@Mock
	private FilterChain filterChain;

	private HttpServletRequest request;
	private MockHttpServletResponse response = new MockHttpServletResponse();


	private static final String credential = "00000000-0000-0000-0000-000000000000";
	private static final String credentialHeader = "X-Labsvar-Credential";

	@Before
	public void setup() throws IOException, ServletException {
		Mockito.when(config.getProperty(SXAConfigKeys.SXA_CREDENTIAL)).thenReturn(credential);
		Mockito.when(config.getProperty(SXAConfigKeys.SXA_CREDENTIAL_HEADER)).thenReturn(credentialHeader);

		request = Mockito.mock(HttpServletRequest.class);
		Mockito.when(request.getHeader(credentialHeader)).thenReturn(credential);

		response.setStatus(HttpStatus.SC_FORBIDDEN);
		Mockito.doAnswer(invocation -> { 
			response.setStatus(HttpStatus.SC_PAYMENT_REQUIRED);
			return null; 
		}).when(filterChain).doFilter(Mockito.any(), Mockito.any());

		subject = Mockito.spy(subject); // makes it possible to query interactions
	}

	@Test
	public void doFilter_success() throws Exception {
		// Arrange
		subject.init(null);

		// Act
		subject.doFilter(request, response, filterChain);

		// Assert
		assertEquals(HttpStatus.SC_PAYMENT_REQUIRED, response.getStatus());
		Mockito.verify(subject, Mockito.times(1)).checkCredential(Mockito.any(), Mockito.any(), Mockito.any());
	}

	@Test
	public void doFilter_configurationOmitted_skipsValidation() throws Exception {
		// Arrange
		Mockito.when(config.getProperty(SXAConfigKeys.SXA_CREDENTIAL)).thenThrow(new ConfigException());
		Mockito.when(config.getProperty(SXAConfigKeys.SXA_CREDENTIAL_HEADER)).thenThrow(new ConfigException());
		subject.init(null);

		// Act
		subject.doFilter(request, response, filterChain);

		// Assert
		assertEquals(HttpStatus.SC_PAYMENT_REQUIRED, response.getStatus());
		Mockito.verify(subject, Mockito.times(0)).checkCredential(Mockito.any(), Mockito.any(), Mockito.any());
	}

	@Test(expected = SoapFault.class)
	public void doFilter_configurationIncomplete_failure() throws Exception {
		// Arrange
		Mockito.when(config.getProperty(SXAConfigKeys.SXA_CREDENTIAL_HEADER)).thenThrow(new ConfigException());
		subject.init(null);

		// Act
		try {
			subject.doFilter(request, response, filterChain);
		} catch (SoapFault e) {
			// Assert
			assertEquals(HttpStatus.SC_FORBIDDEN, response.getStatus());
			Mockito.verify(subject, Mockito.times(0)).checkCredential(Mockito.any(), Mockito.any(), Mockito.any());
			throw e;
		}
	}

	@Test(expected = SoapFault.class)
	public void doFilter_headerMissingFromRequest_failure() throws Exception {
		// Arrange
		Mockito.when(request.getHeader(credentialHeader)).thenReturn(null);
		subject.init(null);

		// Act
		try {
			subject.doFilter(request, response, filterChain);
		} catch (SoapFault e) {
			// Assert
			assertEquals(HttpStatus.SC_FORBIDDEN, response.getStatus());
			Mockito.verify(subject, Mockito.times(1)).checkCredential(Mockito.any(), Mockito.any(), Mockito.any());
			throw e;
		}
	}

	@Test(expected = SoapFault.class)
	public void doFilter_wrongHeader_failure() throws Exception {
		// Arrange
		String cred = "11111111-1111-1111-1111-111111111111";
		Mockito.when(request.getHeader(credentialHeader)).thenReturn(cred);
		subject.init(null);

		// Act
		try {
			subject.doFilter(request, response, filterChain);
		} catch (SoapFault e) {
			// Assert
			assertEquals(HttpStatus.SC_FORBIDDEN, response.getStatus());
			Mockito.verify(subject, Mockito.times(1)).checkCredential(Mockito.any(), Mockito.any(), Mockito.any());
			throw e;
		}
	}

}